@@ -23,6 +23,7 @@
 #include "grn_tokenizers.h"
 #include "grn_ctx_impl.h"
 #include "grn_encoding.h"
+#include "grn_hash.h"
 #include "grn_ii.h"
 #include "grn_pat.h"
 #include "grn_index_column.h"
@@ -163,8 +164,10 @@ grn_init_from_env(void)
   grn_alloc_init_from_env();
   grn_mrb_init_from_env();
   grn_ctx_impl_mrb_init_from_env();
+  grn_hash_init_from_env();
   grn_io_init_from_env();
   grn_ii_init_from_env();
+  grn_ja_init_from_env();
   grn_db_init_from_env();
   grn_expr_init_from_env();
   grn_index_column_init_from_env();
@@ -447,6 +450,8 @@ grn_ctx_impl_init(grn_ctx *ctx)
     goto exit;
   }
 
+  ctx->impl->wal.role = GRN_WAL_ROLE_NONE;
+
   CRITICAL_SECTION_INIT(ctx->impl->children.lock);
   /* grn_obj * isn't grn_ctx * but sizeof(grn_obj *) and
    * sizeof(grn_ctx *) is same. So we reuse GRN_PVECTOR here. */
@@ -1245,6 +1250,27 @@ grn_ctx_set_force_match_escalation(grn_ctx *ctx, grn_bool force)
   }
 }
 
+grn_wal_role
+grn_ctx_get_wal_role(grn_ctx *ctx)
+{
+  if (ctx->impl) {
+    return ctx->impl->wal.role;
+  } else {
+    return GRN_WAL_ROLE_NONE;
+  }
+}
+
+grn_rc
+grn_ctx_set_wal_role(grn_ctx *ctx, grn_wal_role role)
+{
+  if (ctx->impl) {
+    ctx->impl->wal.role = role;
+    return GRN_SUCCESS;
+  } else {
+    return GRN_INVALID_ARGUMENT;
+  }
+}
+
 grn_rc
 grn_ctx_set_variable(grn_ctx *ctx,
                      const char *name,
@@ -2373,7 +2399,7 @@ grn_ctx_output_flush(grn_ctx *ctx, int flags)
   if (ctx->impl->output.arrow_stream_writer) {
     grn_arrow_stream_writer_flush(ctx, ctx->impl->output.arrow_stream_writer);
   }
-  ctx->impl->output.func(ctx, 0, ctx->impl->output.data.ptr);
+  ctx->impl->output.func(ctx, flags, ctx->impl->output.data.ptr);
 }
 
 void
