@@ -238,7 +238,7 @@ grn_io_create_tmp(grn_ctx *ctx, uint32_t header_size, uint32_t segment_size,
     header->flags = flags;
     header->lock = 0;
     grn_memcpy(header->idstr, GRN_IO_IDSTR, 16);
-    if ((io = GRN_MALLOCN(grn_io, 1))) {
+    if ((io = GRN_CALLOC(sizeof(grn_io)))) {
       grn_io_mapinfo *maps = NULL;
       if ((maps = GRN_CALLOC(sizeof(grn_io_mapinfo) * max_segment))) {
         io->header = header;
@@ -343,7 +343,7 @@ grn_io_create(grn_ctx *ctx, const char *path, uint32_t header_size,
         header->lock = 0;
         grn_memcpy(header->idstr, GRN_IO_IDSTR, 16);
         GRN_MSYNC(ctx, &(fis[0]), header, b);
-        if ((io = GRN_MALLOCN(grn_io, 1))) {
+        if ((io = GRN_CALLOC(sizeof(grn_io)))) {
           grn_io_mapinfo *maps = NULL;
           if ((maps = GRN_CALLOC(sizeof(grn_io_mapinfo) * max_segment))) {
             grn_strncpy(io->path, PATH_MAX, path, path_length + 1);
@@ -673,7 +673,7 @@ grn_io_open(grn_ctx *ctx, const char *path, grn_io_mode mode)
 
   grn_fileinfo_init(fis, max_nfiles);
   grn_memcpy(fis, &fi, sizeof(fileinfo));
-  grn_io *io = GRN_MALLOC(sizeof(grn_io));
+  grn_io *io = GRN_CALLOC(sizeof(grn_io));
   if (!io) {
     GRN_FREE(fis);
     GRN_MUNMAP(ctx, &grn_gctx, NULL, &(fi.fmo), &fi, header, b);
@@ -914,13 +914,15 @@ grn_io_remove_if_exist(grn_ctx *ctx, const char *path)
 grn_rc
 grn_io_rename(grn_ctx *ctx, const char *old_name, const char *new_name)
 {
+  const char *tag = "[io][rename]";
   struct stat s;
   if (stat(old_name, &s)) {
-    SERR("failed to stat path to be renamed: <%s>", old_name);
+    SERR("%s failed to stat path to be renamed: <%s>",
+         tag, old_name);
     return ctx->rc;
   } else if (rename(old_name, new_name)) {
-    SERR("failed to rename path: <%s> -> <%s>",
-         old_name, new_name);
+    SERR("%s failed to rename path: <%s> -> <%s>",
+         tag, old_name, new_name);
     return ctx->rc;
   } else {
     int fno;
@@ -928,16 +930,13 @@ grn_io_rename(grn_ctx *ctx, const char *old_name, const char *new_name)
     char new_buffer[PATH_MAX];
     for (fno = 1; ; fno++) {
       gen_pathname(old_name, old_buffer, fno);
-      if (!stat(old_buffer, &s)) {
-        gen_pathname(new_name, new_buffer, fno);
-        if (rename(old_buffer, new_buffer)) {
-          SERR("failed to rename path: <%s> -> <%s>",
-               old_buffer, new_buffer);
-        }
-      } else {
-        SERR("failed to stat path to be renamed: <%s>",
-             old_buffer);
-        return ctx->rc;
+      if (stat(old_buffer, &s) == 0) {
+        break;
+      }
+      gen_pathname(new_name, new_buffer, fno);
+      if (rename(old_buffer, new_buffer)) {
+        SERR("%s failed to rename path: <%s> -> <%s>",
+             tag, old_buffer, new_buffer);
       }
     }
     return GRN_SUCCESS;
@@ -963,7 +962,7 @@ grn_io_read_ja(grn_io *io, grn_ctx *ctx, grn_io_ja_einfo *einfo, uint32_t epos,
   fileinfo *fi = &io->fis[fno];
   off_t base = fno ? 0 : io->base - (uint64_t)segment_size * io->base_seg;
   off_t pos = (uint64_t)segment_size * (bseg % segments_per_file) + offset + base;
-  ja_element *v = GRN_MALLOC(size);
+  ja_element *v = GRN_CALLOC(size);
   if (!v) {
     *value = NULL;
     *value_len = 0;
@@ -1170,7 +1169,7 @@ grn_io_win_map(grn_ctx *ctx,
     iw->cached = 1;
     iw->addr = addr + offset;
   } else {
-    if (!(iw->addr = GRN_MALLOC(size))) { return NULL; }
+    if (!(iw->addr = GRN_CALLOC(size))) { return NULL; }
     iw->cached = 0;
     switch (mode) {
     case GRN_IO_RDONLY:
@@ -2360,3 +2359,40 @@ grn_pwrite(grn_ctx *ctx, fileinfo *fi, void *buf, size_t count, off_t offset)
 }
 
 #endif /* WIN32 */
+
+bool
+grn_io_warm_path(grn_ctx *ctx, grn_io *io, const char *path)
+{
+  FILE *input = grn_fopen(path, "rb");
+  if (!input) {
+    SERR("[io][warm] failed to open a file: <%s>", path);
+    return false;
+  }
+
+  GRN_LOG(ctx, GRN_LOG_DUMP, "[io][warm] <%s>", path);
+  char buffer[4096];
+  while (fread(buffer, 1, sizeof(buffer), input) != 0) {
+    /* Do nothing */
+  }
+  fclose(input);
+  return true;
+}
+
+grn_rc
+grn_io_warm(grn_ctx *ctx, grn_io *io)
+{
+  if (io->path[0] == '\0') {
+    return GRN_SUCCESS;
+  }
+
+  char buffer[PATH_MAX];
+  uint32_t i;
+  uint32_t n_files = grn_io_n_files(ctx, io);
+  for (i = 0; i < n_files; i++) {
+    gen_pathname(io->path, buffer, i);
+    if (!grn_io_warm_path(ctx, io, buffer)) {
+      break;
+    }
+  }
+  return ctx->rc;
+}
