@@ -1,6 +1,6 @@
 /*
   Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -103,6 +103,7 @@ command_table_create(grn_ctx *ctx,
   grn_obj *normalizer_raw;
   grn_raw_string token_filters_raw;
   grn_obj *path_raw;
+  grn_obj *normalizers_raw;
   grn_obj *key_type = NULL;
   grn_obj *value_type = NULL;
   grn_obj *table = NULL;
@@ -126,6 +127,7 @@ command_table_create(grn_ctx *ctx,
   normalizer_raw = grn_plugin_proc_get_var(ctx, user_data, "normalizer", -1);
   GET_VALUE(token_filters);
   path_raw = grn_plugin_proc_get_var(ctx, user_data, "path", -1);
+  normalizers_raw = grn_plugin_proc_get_var(ctx, user_data, "normalizers", -1);
 
 #undef GET_VALUE
 
@@ -216,17 +218,21 @@ command_table_create(grn_ctx *ctx,
       }
     }
 
-    if (GRN_TEXT_LEN(normalizer_raw) > 0) {
-      grn_obj_set_info(ctx, table, GRN_INFO_NORMALIZER, normalizer_raw);
+    if (GRN_TEXT_LEN(normalizers_raw) == 0 &&
+        GRN_TEXT_LEN(normalizer_raw) > 0) {
+      normalizers_raw = normalizer_raw;
+    }
+    if (GRN_TEXT_LEN(normalizers_raw) > 0) {
+      grn_obj_set_info(ctx, table, GRN_INFO_NORMALIZERS, normalizers_raw);
       if (ctx->rc != GRN_SUCCESS) {
         GRN_PLUGIN_ERROR(ctx,
                          GRN_INVALID_ARGUMENT,
                          "[table][create][%.*s] "
-                         "failed to set normalizer: <%.*s>: %s",
+                         "failed to set normalizers: <%.*s>: %s",
                          (int)name_raw.length,
                          name_raw.value,
-                         (int)GRN_TEXT_LEN(normalizer_raw),
-                         GRN_TEXT_VALUE(normalizer_raw),
+                         (int)GRN_TEXT_LEN(normalizers_raw),
+                         GRN_TEXT_VALUE(normalizers_raw),
                          ctx->errbuf);
         goto exit;
       }
@@ -263,7 +269,7 @@ exit :
 void
 grn_proc_init_table_create(grn_ctx *ctx)
 {
-  grn_expr_var vars[8];
+  grn_expr_var vars[9];
 
   grn_plugin_expr_var_init(ctx, &(vars[0]), "name", -1);
   grn_plugin_expr_var_init(ctx, &(vars[1]), "flags", -1);
@@ -273,10 +279,11 @@ grn_proc_init_table_create(grn_ctx *ctx)
   grn_plugin_expr_var_init(ctx, &(vars[5]), "normalizer", -1);
   grn_plugin_expr_var_init(ctx, &(vars[6]), "token_filters", -1);
   grn_plugin_expr_var_init(ctx, &(vars[7]), "path", -1);
+  grn_plugin_expr_var_init(ctx, &(vars[8]), "normalizers", -1);
   grn_plugin_command_create(ctx,
                             "table_create", -1,
                             command_table_create,
-                            8,
+                            9,
                             vars);
 }
 
@@ -324,7 +331,7 @@ output_table_info(grn_ctx *ctx, grn_obj *table)
 
   if (normalizer) {
     GRN_BULK_REWIND(&buffer);
-    grn_table_get_normalizer_string(ctx, table, &buffer);
+    grn_table_get_normalizers_string(ctx, table, &buffer);
     grn_ctx_output_obj(ctx, &buffer, NULL);
   } else {
     grn_ctx_output_null(ctx);
@@ -601,42 +608,12 @@ grn_proc_init_table_rename(grn_ctx *ctx)
                             vars);
 }
 
-static grn_rc
-command_table_copy_resolve_target(grn_ctx *ctx,
-                                  const char *label,
-                                  grn_obj *name,
-                                  grn_obj **table)
-{
-  if (GRN_TEXT_LEN(name) == 0) {
-    GRN_PLUGIN_ERROR(ctx,
-                     GRN_INVALID_ARGUMENT,
-                     "[table][copy] %s name isn't specified",
-                     label);
-    return ctx->rc;
-  }
-  *table = grn_ctx_get(ctx,
-                       GRN_TEXT_VALUE(name),
-                       GRN_TEXT_LEN(name));
-  if (!*table) {
-    GRN_PLUGIN_ERROR(ctx,
-                     GRN_INVALID_ARGUMENT,
-                     "[table][copy] %s table isn't found: <%.*s>",
-                     label,
-                     (int)GRN_TEXT_LEN(name),
-                     GRN_TEXT_VALUE(name));
-    return ctx->rc;
-  }
-
-  return ctx->rc;
-}
-
 static grn_obj *
 command_table_copy(grn_ctx *ctx,
                    int nargs,
                    grn_obj **args,
                    grn_user_data *user_data)
 {
-  grn_rc rc = GRN_SUCCESS;
   grn_obj *from_table = NULL;
   grn_obj *to_table = NULL;
   grn_obj *from_name;
@@ -645,19 +622,19 @@ command_table_copy(grn_ctx *ctx,
   from_name = grn_plugin_proc_get_var(ctx, user_data, "from_name", -1);
   to_name   = grn_plugin_proc_get_var(ctx, user_data, "to_name", -1);
 
-  rc = command_table_copy_resolve_target(ctx, "from", from_name, &from_table);
-  if (rc != GRN_SUCCESS) {
+  from_table = grn_proc_get_value_object(ctx, from_name, "[table][copy][from]");
+  if (!from_table) {
     goto exit;
   }
-  rc = command_table_copy_resolve_target(ctx, "to", to_name, &to_table);
-  if (rc != GRN_SUCCESS) {
+  to_table = grn_proc_get_value_object(ctx, to_name, "[table][copy][to]");
+  if (!to_table) {
     goto exit;
   }
 
-  rc = grn_table_copy(ctx, from_table, to_table);
+  grn_table_copy(ctx, from_table, to_table);
 
 exit :
-  grn_ctx_output_bool(ctx, rc == GRN_SUCCESS);
+  grn_ctx_output_bool(ctx, ctx->rc == GRN_SUCCESS);
 
   if (to_table) {
     grn_obj_unlink(ctx, to_table);
@@ -682,3 +659,57 @@ grn_proc_init_table_copy(grn_ctx *ctx)
                             2,
                             vars);
 }
+
+static grn_obj *
+command_table_create_similar(grn_ctx *ctx,
+                             int nargs,
+                             grn_obj **args,
+                             grn_user_data *user_data)
+{
+  grn_obj *name;
+  grn_obj *base_table_name;
+  grn_obj *base_table = NULL;
+  grn_obj *table = NULL;
+
+  name = grn_plugin_proc_get_var(ctx, user_data, "name", -1);
+  base_table_name = grn_plugin_proc_get_var(ctx, user_data, "base_table", -1);
+
+  base_table = grn_proc_get_value_object(ctx,
+                                         base_table_name,
+                                         "[table][create][similar][base_table]");
+  if (!base_table) {
+    goto exit;
+  }
+
+  table = grn_table_create_similar(ctx,
+                                   GRN_TEXT_VALUE(name),
+                                   GRN_TEXT_LEN(name),
+                                   NULL,
+                                   base_table);
+
+exit :
+  grn_ctx_output_bool(ctx, ctx->rc == GRN_SUCCESS);
+
+  if (base_table) {
+    grn_obj_unref(ctx, base_table);
+  }
+  if (table) {
+    grn_obj_unref(ctx, table);
+  }
+
+  return NULL;
+}
+
+void
+grn_proc_init_table_create_similar(grn_ctx *ctx)
+{
+  grn_expr_var vars[2];
+
+  grn_plugin_expr_var_init(ctx, &(vars[0]), "name", -1);
+  grn_plugin_expr_var_init(ctx, &(vars[1]), "base_table", -1);
+  grn_plugin_command_create(ctx,
+                            "table_create_similar", -1,
+                            command_table_create_similar,
+                            2,
+                            vars);
+}
