@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2017-2018 Brazil
-  Copyright(C) 2018-2020 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2017-2018  Brazil
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -106,6 +106,73 @@ grn_table_module_set_options(grn_ctx *ctx,
   CRITICAL_SECTION_LEAVE(module->lock);
 }
 
+void
+grn_table_modules_init(grn_ctx *ctx,
+                       grn_obj *modules_buffer)
+{
+  GRN_TEXT_INIT(modules_buffer, 0);
+}
+
+void
+grn_table_modules_fin(grn_ctx *ctx,
+                      grn_obj *modules_buffer)
+{
+  grn_table_modules_rewind(ctx, modules_buffer);
+  GRN_OBJ_FIN(ctx, modules_buffer);
+}
+
+void
+grn_table_modules_add(grn_ctx *ctx,
+                      grn_obj *modules_buffer,
+                      grn_obj *proc)
+{
+  grn_bulk_space(ctx, modules_buffer, sizeof(grn_table_module));
+  size_t n = GRN_BULK_VSIZE(modules_buffer) / sizeof(grn_table_module);
+  grn_table_module *module =
+    ((grn_table_module *)GRN_BULK_HEAD(modules_buffer)) + n - 1;
+  grn_table_module_init(ctx, module, grn_obj_id(ctx, proc));
+  grn_table_module_set_proc(ctx, module, proc);
+}
+
+void
+grn_table_modules_rewind(grn_ctx *ctx,
+                         grn_obj *modules_buffer)
+{
+  grn_table_module *modules = (grn_table_module *)GRN_BULK_HEAD(modules_buffer);
+  size_t n = GRN_BULK_VSIZE(modules_buffer) / sizeof(grn_table_module);
+  size_t i;
+  for (i = 0; i < n; i++) {
+    grn_table_module_fin(ctx, modules + i);
+  }
+  GRN_BULK_REWIND(modules_buffer);
+}
+
+grn_table_module *
+grn_table_modules_get_module(grn_ctx *ctx,
+                             grn_obj *modules_buffer,
+                             size_t i)
+{
+  grn_table_module *modules = (grn_table_module *)GRN_BULK_HEAD(modules_buffer);
+  size_t n = GRN_BULK_VSIZE(modules_buffer) / sizeof(grn_table_module);
+  if (i >= n) {
+    return NULL;
+  }
+  return modules + i;
+}
+
+grn_obj *
+grn_table_modules_get_proc(grn_ctx *ctx,
+                           grn_obj *modules_buffer,
+                           size_t i)
+{
+  grn_table_module *module =
+    grn_table_modules_get_module(ctx, modules_buffer, i);
+  if (!module) {
+    return NULL;
+  }
+  return module->proc;
+}
+
 static grn_rc
 grn_table_set_module_options(grn_ctx *ctx,
                              grn_obj *table,
@@ -187,7 +254,7 @@ typedef struct {
   const char *context_tag;
   const char *module_name;
   grn_info_type type;
-  unsigned int token_filter_index;
+  size_t index;
   grn_table_module_open_options_func open_options_func;
   grn_close_func close_options_func;
   void *user_data;
@@ -232,13 +299,19 @@ grn_table_cache_module_options(grn_ctx *ctx,
   case GRN_INFO_NORMALIZER :
     switch (table->header.type) {
     case GRN_TABLE_HASH_KEY :
-      module = &(((grn_hash *)table)->normalizer);
+      module = grn_table_modules_get_module(ctx,
+                                            &(((grn_hash *)table)->normalizers),
+                                            0);
       break;
     case GRN_TABLE_PAT_KEY :
-      module = &(((grn_pat *)table)->normalizer);
+      module = grn_table_modules_get_module(ctx,
+                                            &(((grn_pat *)table)->normalizers),
+                                            0);
       break;
     case GRN_TABLE_DAT_KEY :
-      module = &(((grn_dat *)table)->normalizer);
+      module = grn_table_modules_get_module(ctx,
+                                            &(((grn_dat *)table)->normalizers),
+                                            0);
       break;
     default :
       break;
@@ -262,7 +335,28 @@ grn_table_cache_module_options(grn_ctx *ctx,
       }
       module =
         ((grn_table_module *)GRN_BULK_HEAD(token_filters)) +
-        data->token_filter_index;
+        data->index;
+    }
+    break;
+  case GRN_INFO_NORMALIZERS :
+    {
+      grn_obj *normalizers = NULL;
+      switch (table->header.type) {
+      case GRN_TABLE_HASH_KEY :
+        normalizers = &(((grn_hash *)table)->normalizers);
+        break;
+      case GRN_TABLE_PAT_KEY :
+        normalizers = &(((grn_pat *)table)->normalizers);
+        break;
+      case GRN_TABLE_DAT_KEY :
+        normalizers = &(((grn_dat *)table)->normalizers);
+        break;
+      default :
+        break;
+      }
+      module =
+        ((grn_table_module *)GRN_BULK_HEAD(normalizers)) +
+        data->index;
     }
     break;
   default :
@@ -270,30 +364,32 @@ grn_table_cache_module_options(grn_ctx *ctx,
   }
 
   GRN_TEXT_INIT(&raw_options, GRN_OBJ_VECTOR);
-  CRITICAL_SECTION_ENTER(module->lock);
-  revision = grn_obj_get_option_values(ctx,
-                                       table,
-                                       data->module_name,
-                                       -1,
-                                       module->options_revision,
-                                       &raw_options);
-  bool need_update = true;
-  if (revision == GRN_OPTION_REVISION_UNCHANGED) {
-    need_update = false;
-  }
-  if (revision == GRN_OPTION_REVISION_NONE && module->options) {
-    need_update = false;
-  }
-  if (need_update) {
-    options = data->open_options_func(ctx,
-                                      module->proc,
-                                      &raw_options,
-                                      data->user_data);
-    grn_table_module_set_options_without_lock(ctx,
-                                              module,
-                                              options,
-                                              revision,
-                                              data->close_options_func);
+  if (module) {
+    CRITICAL_SECTION_ENTER(module->lock);
+    revision = grn_obj_get_option_values(ctx,
+                                         table,
+                                         data->module_name,
+                                         -1,
+                                         module->options_revision,
+                                         &raw_options);
+    bool need_update = true;
+    if (revision == GRN_OPTION_REVISION_UNCHANGED) {
+      need_update = false;
+    }
+    if (revision == GRN_OPTION_REVISION_NONE && module->options) {
+      need_update = false;
+    }
+    if (need_update) {
+      options = data->open_options_func(ctx,
+                                        module->proc,
+                                        &raw_options,
+                                        data->user_data);
+      grn_table_module_set_options_without_lock(ctx,
+                                                module,
+                                                options,
+                                                revision,
+                                                data->close_options_func);
+    }
   }
   CRITICAL_SECTION_LEAVE(module->lock);
   GRN_OBJ_FIN(ctx, &raw_options);
@@ -357,6 +453,77 @@ grn_table_get_module_string_raw(grn_ctx *ctx,
   GRN_OBJ_FIN(ctx, &options);
 }
 
+static bool
+grn_table_using_normalizer(grn_ctx *ctx, grn_obj *table)
+{
+  grn_id proc_id = GRN_ID_NIL;
+  switch (table->header.type) {
+  case GRN_TABLE_HASH_KEY :
+    proc_id = ((grn_hash *)table)->header.common->normalizer;
+    break;
+  case GRN_TABLE_PAT_KEY :
+    proc_id = ((grn_pat *)table)->header->normalizer;
+    break;
+  case GRN_TABLE_DAT_KEY :
+    proc_id = ((grn_dat *)table)->header->normalizer;
+    break;
+  default :
+    break;
+  }
+  return proc_id != GRN_ID_NIL;
+}
+
+static void
+grn_table_module_format_name(
+  grn_ctx *ctx,
+  grn_obj *table,
+  const char *module_name,
+  uint32_t i,
+  grn_info_type type,
+  bool for_get,
+  char real_module_name[GRN_TABLE_MAX_KEY_SIZE])
+{
+  bool need_index = false;
+  switch (type) {
+  case GRN_INFO_NORMALIZER :
+    if (for_get) {
+      need_index = !grn_table_using_normalizer(ctx, table);
+    }
+    break;
+  case GRN_INFO_NORMALIZERS :
+    if (for_get) {
+      if (i == 0) {
+        need_index = !grn_table_using_normalizer(ctx, table);
+      } else {
+        need_index = true;
+      }
+    } else {
+      need_index = true;
+    }
+    break;
+  case GRN_INFO_TOKEN_FILTERS :
+    need_index = true;
+    break;
+  default :
+    break;
+  }
+
+  if (need_index) {
+    grn_snprintf(real_module_name,
+                 GRN_TABLE_MAX_KEY_SIZE,
+                 GRN_TABLE_MAX_KEY_SIZE,
+                 "%s%u",
+                 module_name,
+                 i);
+  } else {
+    grn_snprintf(real_module_name,
+                 GRN_TABLE_MAX_KEY_SIZE,
+                 GRN_TABLE_MAX_KEY_SIZE,
+                 "%s",
+                 module_name);
+  }
+}
+
 static grn_rc
 grn_table_get_module_string(grn_ctx *ctx,
                             grn_obj *table,
@@ -427,12 +594,13 @@ grn_table_get_modules_string(grn_ctx *ctx,
     if (i > 0) {
       GRN_TEXT_PUTS(ctx, output, ", ");
     }
-    grn_snprintf(real_module_name,
-                 GRN_TABLE_MAX_KEY_SIZE,
-                 GRN_TABLE_MAX_KEY_SIZE,
-                 "%s%u",
-                 module_name,
-                 i);
+    grn_table_module_format_name(ctx,
+                                 table,
+                                 module_name,
+                                 i,
+                                 type,
+                                 true,
+                                 real_module_name);
     grn_table_get_module_string_raw(ctx,
                                     table,
                                     output,
@@ -518,9 +686,17 @@ grn_table_get_normalizer_options(grn_ctx *ctx,
                                  grn_obj *table,
                                  grn_obj *options)
 {
+  char module_name[GRN_TABLE_MAX_KEY_SIZE];
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_NORMALIZER,
+                               0,
+                               GRN_INFO_NORMALIZER,
+                               true,
+                               module_name);
   return grn_table_get_module_options(ctx,
                                       table,
-                                      OPTION_NAME_NORMALIZER,
+                                      module_name,
                                       options,
                                       "normalizer");
 }
@@ -528,7 +704,6 @@ grn_table_get_normalizer_options(grn_ctx *ctx,
 void *
 grn_table_cache_normalizer_options(grn_ctx *ctx,
                                    grn_obj *table,
-                                   /* TODO: Remove me. */
                                    grn_obj *string,
                                    grn_table_module_open_options_func open_options_func,
                                    grn_close_func close_options_func,
@@ -538,7 +713,16 @@ grn_table_cache_normalizer_options(grn_ctx *ctx,
 
   memset(&data, 0, sizeof(data));
   data.context_tag = "normalizer";
-  data.module_name = OPTION_NAME_NORMALIZER;
+  uint32_t i = grn_string_get_normalizer_index(ctx, string);
+  char module_name[GRN_TABLE_MAX_KEY_SIZE];
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_NORMALIZER,
+                               i,
+                               GRN_INFO_NORMALIZERS,
+                               true,
+                               module_name);
+  data.module_name = module_name;
   data.type = GRN_INFO_NORMALIZER;
   data.open_options_func = open_options_func;
   data.close_options_func = close_options_func;
@@ -551,32 +735,135 @@ grn_table_get_normalizer_string(grn_ctx *ctx,
                                 grn_obj *table,
                                 grn_obj *output)
 {
+  char module_name[GRN_TABLE_MAX_KEY_SIZE];
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_NORMALIZER,
+                               0,
+                               GRN_INFO_NORMALIZER,
+                               true,
+                               module_name);
   return grn_table_get_module_string(ctx,
                                      table,
                                      output,
                                      GRN_INFO_NORMALIZER,
-                                     OPTION_NAME_NORMALIZER,
+                                     module_name,
                                      "normalizer");
 }
 
+grn_rc
+grn_table_set_normalizers_options(grn_ctx *ctx,
+                                  grn_obj *table,
+                                  uint32_t i,
+                                  grn_obj *options)
+{
+  char module_name[GRN_TABLE_MAX_KEY_SIZE];
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_NORMALIZER,
+                               i,
+                               GRN_INFO_NORMALIZERS,
+                               false,
+                               module_name);
+  return grn_table_set_module_options(ctx,
+                                      table,
+                                      module_name,
+                                      options,
+                                      "normalizers");
+}
+
+grn_rc
+grn_table_get_normalizers_options(grn_ctx *ctx,
+                                  grn_obj *table,
+                                  uint32_t i,
+                                  grn_obj *options)
+{
+  char module_name[GRN_TABLE_MAX_KEY_SIZE];
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_NORMALIZER,
+                               i,
+                               GRN_INFO_NORMALIZER,
+                               true,
+                               module_name);
+  return grn_table_get_module_options(ctx,
+                                      table,
+                                      module_name,
+                                      options,
+                                      "normalizers");
+}
+
+void *
+grn_table_cache_normalizers_options(
+  grn_ctx *ctx,
+  grn_obj *table,
+  uint32_t i,
+  grn_table_module_open_options_func open_options_func,
+  grn_close_func close_options_func,
+  void *user_data)
+{
+  grn_table_cache_data data;
+  char module_name[GRN_TABLE_MAX_KEY_SIZE];
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_NORMALIZER,
+                               i,
+                               GRN_INFO_NORMALIZERS,
+                               true,
+                               module_name);
+
+  memset(&data, 0, sizeof(data));
+  data.context_tag = "normalizers";
+  data.module_name = module_name;
+  data.type = GRN_INFO_NORMALIZERS;
+  data.index = i;
+  data.open_options_func = open_options_func;
+  data.close_options_func = close_options_func;
+  data.user_data = user_data;
+  return grn_table_cache_module_options(ctx, table, &data);
+}
+
+grn_rc
+grn_table_get_normalizers_string(grn_ctx *ctx,
+                                 grn_obj *table,
+                                 grn_obj *output)
+{
+  return grn_table_get_modules_string(ctx,
+                                      table,
+                                      output,
+                                      GRN_INFO_NORMALIZERS,
+                                      OPTION_NAME_NORMALIZER,
+                                      "normalizers");
+}
+
 grn_rc
 grn_table_set_token_filter_options(grn_ctx *ctx,
                                    grn_obj *table,
                                    unsigned int i,
                                    grn_obj *options)
+{
+  return grn_table_set_token_filters_options(ctx, table, i, options);
+}
+
+grn_rc
+grn_table_set_token_filters_options(grn_ctx *ctx,
+                                    grn_obj *table,
+                                    uint32_t i,
+                                    grn_obj *options)
 {
   char module_name[GRN_TABLE_MAX_KEY_SIZE];
-  grn_snprintf(module_name,
-               GRN_TABLE_MAX_KEY_SIZE,
-               GRN_TABLE_MAX_KEY_SIZE,
-               "%s%u",
-               OPTION_NAME_TOKEN_FILTER,
-               i);
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_TOKEN_FILTER,
+                               i,
+                               GRN_INFO_TOKEN_FILTERS,
+                               false,
+                               module_name);
   return grn_table_set_module_options(ctx,
                                       table,
                                       module_name,
                                       options,
-                                      "token-filter");
+                                      "token-filters");
 }
 
 grn_rc
@@ -584,43 +871,68 @@ grn_table_get_token_filter_options(grn_ctx *ctx,
                                    grn_obj *table,
                                    unsigned int i,
                                    grn_obj *options)
+{
+  return grn_table_get_token_filters_options(ctx, table, i, options);
+}
+
+grn_rc
+grn_table_get_token_filters_options(grn_ctx *ctx,
+                                    grn_obj *table,
+                                    uint32_t i,
+                                    grn_obj *options)
 {
   char module_name[GRN_TABLE_MAX_KEY_SIZE];
-  grn_snprintf(module_name,
-               GRN_TABLE_MAX_KEY_SIZE,
-               GRN_TABLE_MAX_KEY_SIZE,
-               "%s%u",
-               OPTION_NAME_TOKEN_FILTER,
-               i);
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_TOKEN_FILTER,
+                               i,
+                               GRN_INFO_TOKEN_FILTERS,
+                               true,
+                               module_name);
   return grn_table_get_module_options(ctx,
                                       table,
                                       module_name,
                                       options,
-                                      "token-filter");
+                                      "token-filters");
 }
 
 void *
-grn_table_cache_token_filter_options(grn_ctx *ctx,
-                                     grn_obj *table,
-                                     unsigned int i,
-                                     grn_table_module_open_options_func open_options_func,
-                                     grn_close_func close_options_func,
-                                     void *user_data)
+grn_table_cache_token_filter_options(
+  grn_ctx *ctx,
+  grn_obj *table,
+  unsigned int i,
+  grn_table_module_open_options_func open_options_func,
+  grn_close_func close_options_func,
+  void *user_data)
+{
+  return grn_table_cache_token_filters_options(
+    ctx, table, i, open_options_func, close_options_func, user_data);
+}
+
+void *
+grn_table_cache_token_filters_options(
+  grn_ctx *ctx,
+  grn_obj *table,
+  uint32_t i,
+  grn_table_module_open_options_func open_options_func,
+  grn_close_func close_options_func,
+  void *user_data)
 {
   grn_table_cache_data data;
   char module_name[GRN_TABLE_MAX_KEY_SIZE];
-  grn_snprintf(module_name,
-               GRN_TABLE_MAX_KEY_SIZE,
-               GRN_TABLE_MAX_KEY_SIZE,
-               "%s%u",
-               OPTION_NAME_TOKEN_FILTER,
-               i);
+  grn_table_module_format_name(ctx,
+                               table,
+                               OPTION_NAME_TOKEN_FILTER,
+                               i,
+                               GRN_INFO_TOKEN_FILTERS,
+                               true,
+                               module_name);
 
   memset(&data, 0, sizeof(data));
   data.context_tag = "token-filter";
   data.module_name = module_name;
   data.type = GRN_INFO_TOKEN_FILTERS;
-  data.token_filter_index = i;
+  data.index = i;
   data.open_options_func = open_options_func;
   data.close_options_func = close_options_func;
   data.user_data = user_data;
@@ -637,5 +949,5 @@ grn_table_get_token_filters_string(grn_ctx *ctx,
                                       output,
                                       GRN_INFO_TOKEN_FILTERS,
                                       OPTION_NAME_TOKEN_FILTER,
-                                      "token-filter");
+                                      "token-filters");
 }
